@@ -12,288 +12,287 @@
 # PURPOSE, OR NON-INFRINGEMENT.  MAJORMODE SHALL NOT BE LIABLE FOR ANY
 # LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 # OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
+import uuid
 
 from majormode.perseus.constant.http import HttpMethod
-from majormode.perseus.constant.obj import ObjectStatus
 from majormode.perseus.model.geolocation import GeoPoint
 from majormode.perseus.service.base_http_handler import HttpRequest
 from majormode.perseus.service.base_http_handler import HttpRequestHandler
 from majormode.perseus.service.base_http_handler import http_request
-from majormode.xeberus.model.tracker import BatteryStateChangeEvent
-from majormode.xeberus.model.tracker import LocationUpdate
+from majormode.perseus.utils import cast
+from majormode.xeberus.model.device import BatteryStateChangeEvent
+from majormode.xeberus.model.device import LocationUpdate
 
 from majormode.xeberus.service.device.device_service import DeviceService
 
 
 class DeviceServiceHttpRequestHandler(HttpRequestHandler):
-    @http_request(r'^/device/handshake$',
+    @http_request(r'^/device/(device_id:uuid)/activation$',
                   http_method=HttpMethod.POST,
                   authentication_required=False,
                   signature_required=True)
-    def shake_hands(self, request: HttpRequest, device_id):
-        # Check whether the information of the agent application has been
-        # correctly passed.
-        if not request.client_application:
-            raise DeviceService.InvalidOperationException(
-                "The information of the device agent application has not be given (cf. User-Agent HTTP header)")
+    def activate_device_app(self, request, device_id):
+        activation_code = request.get_argument(
+            'activation_code',
+            data_type=HttpRequest.ArgumentDataType.string,
+            is_required=True
+        )
 
         location = request.get_argument(
             'location',
             data_type=HttpRequest.ArgumentDataType.object,
             object_class=GeoPoint,
-            is_required=False)
+            is_required=False
+        )
 
-        mac_address = request.get_argument(
-            'mac_address',
-            data_type=HttpRequest.ArgumentDataType.macaddr,
-            is_required=True)
-
-        serial_number = request.get_argument(
-            'serial_number',
-            data_type=HttpRequest.ArgumentDataType.string,
-            is_required=True)
-
-        return DeviceService().shake_hands(
+        return DeviceService().activate_device_app(
             request.app_id,
-            request.client_application,
-            request.client_ip_address,
-            mac_address,
-            location=location,
-            serial_number=serial_number)
+            device_id,
+            activation_code
+        )
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-    @http_request(r'^/device/(device_id)/status$',
-                  http_method=HttpMethod.PUT,
+    @http_request(r'^/device/activation/code',
+                  http_method=HttpMethod.POST,
                   authentication_required=True,
                   signature_required=True)
-    def set_device_status(self, request, device_id):
-        status = request.get_argument(
-            'status',
-            data_type=HttpRequest.ArgumentDataType.enumeration,
-            enumeration=ObjectStatus,
-            is_required=True)
+    def generate_activation_code(self, request):
+        activation_code_ttl = request.get_argument(
+            'ttl',
+            data_type=HttpRequest.ArgumentDataType.integer,
+            is_required=False
+        )
 
-        # The identification of an organization is only required if the device
-        # is controlled by an organization.
         team_id = request.get_argument(
             'team_id',
-            data_type=HttpRequest.ArgumentDataType.string,
-            is_required=False)
+            data_type=HttpRequest.ArgumentDataType.uuid,
+            is_required=False
+        )
 
-        return DeviceService.set_device_status(
+        return DeviceService().generate_activation_code(
             request.app_id,
-            request.account_id,
-            device_id,
-            status,
-            team_id=team_id)
+            request.session.account_id,
+            activation_code_ttl=activation_code_ttl,
+            team_id=team_id
+        )
 
-
-
-
-
-
-
-
-
-
-
-    @http_request(r'^/device/(device_id)/activation/(activation_code)$',
-                  http_method=HttpMethod.POST,
-                  authentication_required=False,
-                  signature_required=True)
-    def activate_device(self, request, device_id, activation_code):
-        # Check whether the information of the agent application has been
-        # correctly passed.
-        if not request.client_application:
-            raise DeviceService.InvalidOperationException(
-                "the information of the device agent application has not be given (cf. User-Agent HTTP header)")
-
-        serial_number = request.get_argument(
-            'serial_number',
-            data_type=HttpRequest.ArgumentDataType.string,
-            is_required=True)
-
-        mac_address = request.get_argument(
-            'mac_address',
-            data_type=HttpRequest.ArgumentDataType.macaddr,
-            is_required=True)
-
-        battery_level = request.get_argument(
-            'battery_level',
-            data_type=HttpRequest.ArgumentDataType.decimal,
-            is_required=True)
-
-        is_battery_plugged = request.get_argument(
-            'is_battery_plugged',
-            data_type=HttpRequest.ArgumentDataType.boolean,
-            is_required=True)
-
-        location = request.get_argument(
-            'location',
-            data_type=HttpRequest.ArgumentDataType.object,
-            object_class=GeoPoint,
-            is_required=False)
-
-        return DeviceService().activate_device(
-            request.app_id,
-            activation_code,
-            device_id,
-            serial_number,
-            mac_address,
-            request.client_application,
-            battery_level,
-            is_battery_plugged,
-            location)
-
-    @http_request(r'^/device/(device_id)/battery/event$',
+    @http_request(r'^/device/(device_id:uuid)/battery/event$',
                   http_method=HttpMethod.POST,
                   authentication_required=False,
                   signature_required=True)
     def report_battery_events(self, request, device_id):
-        if not isinstance(request.body, list):
-            raise DeviceService.InvalidArgumentException("The message body MUST be a list of battery events")
+        events = request.get_argument(
+            'events',
+            data_type=HttpRequest.ArgumentDataType.list,
+            item_data_type=HttpRequest.ArgumentDataType.object,
+            object_class=BatteryStateChangeEvent,
+            is_required=True
+        )
 
-        seed = request.get_argument(
-            's',
+        token = request.get_argument(
+            'token',
             data_type=HttpRequest.ArgumentDataType.string,
-            argument_passing=HttpRequest.ArgumentPassing.query_string,
-            is_required=False)
-
-        events = [
-            BatteryStateChangeEvent.from_json(payload)
-            for payload in request.body
-        ]
+            is_required=True
+        )
 
         return DeviceService().report_battery_events(
             request.app_id,
             device_id,
             events,
-            account_id=request.session and request.session.account_id,
-            seed=seed)
+            token,
+            account_id=request.session and request.session.account_id
+        )
 
-    @http_request(r'^/device/(device_id)/keepalive$',
+    @http_request(r'^/device/(device_id:uuid)/keepalive/event$',
                   http_method=HttpMethod.POST,
                   authentication_required=False,
                   signature_required=True)
-    def report_keepalive(self, request, device_id):
-        seed = request.get_argument(
-            's',
-            data_type=HttpRequest.ArgumentDataType.string,
-            argument_passing=HttpRequest.ArgumentPassing.query_string,
-            is_required=True)
-
-        battery_level = request.get_argument(
-            'battery_level',
-            data_type=HttpRequest.ArgumentDataType.decimal,
-            is_required=False)
-
-        is_battery_plugged = request.get_argument(
-            'is_battery_plugged',
-            data_type=HttpRequest.ArgumentDataType.boolean,
-            is_required=False)
+    def report_keepalive_event(self, request, device_id):
+        event_time = request.get_argument(
+            'event_time',
+            data_type=HttpRequest.ArgumentDataType.timestamp,
+            is_required=True
+        )
 
         location = request.get_argument(
             'location',
             data_type=HttpRequest.ArgumentDataType.object,
             object_class=GeoPoint,
-            is_required=False)
+            is_required=False
+        )
 
         network_type = request.get_argument(
             'network_type',
             data_type=HttpRequest.ArgumentDataType.string,
-            is_required=False)
+            is_required=False
+        )
 
         mobile_exchanged_byte_count = request.get_argument(
             'ex_bytes',
-            data_type=HttpRequest.ArgumentDataType.integer,
-            is_required=False)
+            data_type=HttpRequest.ArgumentDataType.string,
+            is_required=False
+        )
 
-        return DeviceService().report_keepalive(
+        token = request.get_argument(
+            'token',
+            data_type=HttpRequest.ArgumentDataType.string,
+            is_required=True
+        )
+
+        return DeviceService().report_keepalive_event(
             request.app_id,
             device_id,
-            seed,
+            event_time,
+            token,
             account_id=request.session and request.session.account_id,
-            battery_level=battery_level,
-            is_battery_plugged=is_battery_plugged,
             location=location,
-            mobile_exchanged_byte_count=mobile_exchanged_byte_count,
-            network_type=network_type)
+            network_type=network_type
+        )
 
-    @http_request(r'^/device/(device_id)/location$',
+    @http_request(r'^/device/(device_id:uuid)/location/event$',
                   http_method=HttpMethod.POST,
                   authentication_required=False,
                   signature_required=True)
-    def report_location_updates(self, request, device_id):
-        if not isinstance(request.body, list):
-            raise DeviceService.InvalidArgumentException("The message body MUST be a list of location updates")
+    def report_location_events(self, request, device_id):
+        events = request.get_argument(
+            'events',
+            data_type=HttpRequest.ArgumentDataType.list,
+            item_data_type = HttpRequest.ArgumentDataType.object,
+            object_class=LocationUpdate,
+            is_required=True)
 
-        seed = request.get_argument(
-            's',
+        token = request.get_argument(
+            'token',
             data_type=HttpRequest.ArgumentDataType.string,
-            argument_passing=HttpRequest.ArgumentPassing.query_string,
-            is_required=False)
+            is_required=True)
 
-        team_id = request.get_argument(
-            'team_id',
-            data_type=HttpRequest.ArgumentDataType.string,
-            argument_passing=HttpRequest.ArgumentPassing.query_string,
-            is_required=False)
-
-        events = [
-            LocationUpdate.from_json(payload, device_id)
-            for payload in request.body
-        ]
-
-        return DeviceService().report_location_updates(
+        return DeviceService().report_location_events(
             request.app_id,
             device_id,
             events,
-            account_id=request.session and request.session.account_id,
-            seed=seed,
-            team_id=team_id)
+            token,
+            account_id=request.session and request.session.account_id)
 
-    @http_request(r'^/device/activation/request$',
+    @http_request(r'^/device/(device_id:string)/handshake$',
                   http_method=HttpMethod.POST,
-                  authentication_required=True,
+                  authentication_required=False,
                   signature_required=True)
-    def request_activation_code(self, request):
-        activation_code_duration = request.get_argument(
-            'duration',
-            data_type=HttpRequest.ArgumentDataType.integer,
-            is_required=False)
+    def shake_hands(self, request: HttpRequest, device_id: uuid.UUID):
+        # Check whether the information of the agent application has been
+        # correctly passed.
+        if not request.client_application:
+            raise DeviceService.InvalidOperationException(
+                "The information of the device agent application has not be given (cf. User-Agent HTTP header)"
+            )
 
-        team_id = request.get_argument(
-            'team_id',
-            data_type=HttpRequest.ArgumentDataType.uuid,
-            is_required=True)
+        location = request.get_argument(
+            'location',
+            data_type=HttpRequest.ArgumentDataType.object,
+            object_class=GeoPoint,
+            is_required=False
+        )
 
-        return DeviceService().request_activation_code(
+        # @deprecated: MAC address is not available with new version of Android
+        #     in 2021 (cf. Xiaomi), even with programmatic tricks.
+        mac_address = cast.macaddr_to_string(
+            request.get_argument(
+                'mac_address',
+                data_type=HttpRequest.ArgumentDataType.macaddr,
+                is_required=True
+            ),
+            strict=False
+        )
+
+        # @deprecated: Serial number is not available since Android 10.  This
+        #     version changes the permissions for device identifiers so that all
+        #     device identifiers are now protected by the
+        #     `READ_PRIVILEGED_PHONE_STATE` permission.  This permission is only
+        #     granted to apps signed with the platform key and privileged system
+        #     apps.
+        serial_number = request.get_argument(
+            'serial_number',
+            data_type=HttpRequest.ArgumentDataType.string,
+            is_required=True
+        )
+
+        return DeviceService().shake_hands(
             request.app_id,
-            request.account_id,
-            team_id,
-            activation_code_duration=activation_code_duration)
+            device_id,
+            request.client_application,
+            request.client_ip_address,
+            location=location,
+            mac_address=mac_address,
+            serial_number=serial_number
+        )
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+    # @http_request(r'^/device/(device_id)/status$',
+    #               http_method=HttpMethod.PUT,
+    #               authentication_required=True,
+    #               signature_required=True)
+    # def set_device_status(self, request, device_id):
+    #     status = request.get_argument(
+    #         'status',
+    #         data_type=HttpRequest.ArgumentDataType.enumeration,
+    #         enumeration=ObjectStatus,
+    #         is_required=True)
+    #
+    #     # The identification of an organization is only required if the device
+    #     # is controlled by an organization.
+    #     team_id = request.get_argument(
+    #         'team_id',
+    #         data_type=HttpRequest.ArgumentDataType.string,
+    #         is_required=False)
+    #
+    #     return DeviceService.set_device_status(
+    #         request.app_id,
+    #         request.account_id,
+    #         device_id,
+    #         status,
+    #         team_id=team_id)
+
+
+
+
+
+
+
+
+
+
+
 
 
 
